@@ -1,22 +1,22 @@
-/* matchbox - a lightweight window manager
-
-   Copyright 2002 Matthew Allum
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-*/
-
-/*
-  $Id: base_client.c,v 1.7 2004/09/24 11:05:16 mallum Exp $
-*/
-
+/* 
+ *  Matchbox Window Manager - A lightweight window manager not for the
+ *                            desktop.
+ *
+ *  Authored By Matthew Allum <mallum@o-hand.com>
+ *
+ *  Copyright (c) 2002, 2004 OpenedHand Ltd - http://o-hand.com
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
 
 #include "base_client.h"
 
@@ -25,47 +25,49 @@ base_client_new(Wm *w, Window win)
 {
    XWindowAttributes attr;
 
-   Client *c = NULL;
-   int i = 0, n = 0, format;
-   XWMHints *wmhints = NULL;
-   Atom *protocols   = NULL;
+   Client        *c = NULL;
+   int            i = 0, n = 0, format;
+   XWMHints      *wmhints = NULL;
+   Atom          *protocols = NULL;
    XTextProperty  text_prop;
-
-   Atom type;
-   long bytes_after, n_items;
-   long *data = NULL;
+   Atom           type;
+   long           bytes_after, n_items, *data = NULL;
+   unsigned long  val[1];
 
    dbg("%s() called  \n", __func__);
 
    c = malloc(sizeof(Client));
    memset(c, 0, sizeof(Client));
-   
-   c->type    = mainwin; /* start off with common case */
-   c->buttons = NULL;
-   c->name    = NULL;
 
-   XGetWindowAttributes(w->dpy, win, &attr);
-  
-   c->window = win;
-   c->frame       = None;
-   c->title_frame = None;
-   c->have_set_bg = False;
-
-   c->wm = w;
-   c->ignore_unmap = 0;
+   /* Stardard bits */
    
+   c->type    = MBCLIENT_TYPE_APP; /* start off with common case */
+   c->window  = win;
+   c->wm      = w;
+
+   /* Set up the 'methods' - expect to be overidden */
+
+   base_client_set_funcs(c);
+
+   stack_prepend_bottom(c); 
+
+   for (i=0; i<MSK_COUNT; i++)
+     c->backing_masks[i] = None;
+
+   for (i=0; i<N_DECOR_FRAMES; i++)
+     c->frames_decor[i] = None;
+
+   /* UTF8 Window Name */
+
    if ((c->name = ewmh_get_utf8_prop(w, win, w->atoms[_NET_WM_NAME])) != NULL)
      c->name_is_utf8 = True;
 
-   c->subname = ewmh_get_utf8_prop(w, win, w->atoms[MB_WIN_SUB_NAME]);
+   /* Basic attributes */
 
-   base_client_process_name(c);
-   
-   c->x = attr.x;
-   c->y = attr.y;
-
-   c->have_cache = False;
+   XGetWindowAttributes(w->dpy, win, &attr);
    
+   c->x      = attr.x;
+   c->y      = attr.y;
    c->cmap   = attr.colormap;
    c->visual = attr.visual;
 
@@ -81,16 +83,20 @@ base_client_new(Wm *w, Window win)
 				&set_attrs);
      }
 
-   /* Get size hints */
-   c->size = XAllocSizeHints();
-
-   c->width = attr.width;
+   c->width  = attr.width;
    c->height = attr.height;
 
+   if (c->x < 0) c->x = (w->dpy_width + c->x - c->width);
+   if (c->y < 0) c->y = (w->dpy_height + c->y - c->height);
+
 #if 0
 
+   /* Get size hints */
+
+   c->size = XAllocSizeHints();
+
    /* 
-    * Dont bother with WMNOrmalHints anymore, reasoning;
+    * Dont bother with WMNormalHints anymore, reasoning;
     * 
     * - mb forces app window sizes anyway ( and static )
     * - We dont let users resize dialogs 
@@ -120,76 +126,65 @@ base_client_new(Wm *w, Window win)
    }
 #endif
 
-   if (c->x < 0) c->x = (w->dpy_width + c->x - c->width);
-       
-   if (c->y < 0) c->y = (w->dpy_height + c->y - c->height);
+   /* EWMH Icon */
+
+#ifndef STANDALONE
+   c->icon_rgba_data = ewmh_get_icon_prop_data(w, win);
+#endif
+
+   /* WM Hints */
 
    if ((wmhints = XGetWMHints(w->dpy, c->window)) != NULL)
    {
+     dbg("%s() checking WMHints\n", __func__);
+
       if (wmhints->window_group)
 	 c->win_group = wmhints->window_group;
       else
 	 c->win_group = 0;
+
+      if (wmhints->flags & XUrgencyHint)
+	{
+	  dbg("%s() WMHints, has urgency hint\n", __func__);
+	  c->flags |= CLIENT_HAS_URGENCY_FLAG;
+	}
+
+      c->icon = None;
+      c->icon_mask = None;
+
+      if (w->config->use_icons && c->icon_rgba_data == NULL)
+	{
+	  if (wmhints->flags & IconPixmapHint)
+	    {
+	      dbg("%s() got icon hint\n", __func__); 
+	      c->icon = wmhints->icon_pixmap;
+	      if (wmhints->flags & IconMaskHint)
+		{
+		  c->icon_mask = wmhints->icon_mask;
+		}
+	    }
+	}
    }
+
    dbg("%s() window group %li\n", __func__, c->win_group);
 
+   if (wmhints) XFree(wmhints);
+
+
+#ifdef USE_LIBSN
+
+   /* Startup Notification */
+
    c->startup_id = ewmh_get_utf8_prop(w, win, w->atoms[_NET_STARTUP_ID]);
 
    if (c->startup_id == NULL && c->win_group)
      c->startup_id = ewmh_get_utf8_prop(w, c->win_group, 
 					w->atoms[_NET_STARTUP_ID]);
 
-   c->bin_name = NULL;
+#endif
      
-   XFree(wmhints);
+   /* Where is client running ? */
 
-   /* Set up the 'methods' - expect to be overidden */
-   base_client_set_funcs(c);
-
-   /* Add the client to the circular list */
-   if (w->head_client == NULL)
-   {
-      c->next = c;
-      c->prev = c;
-      w->head_client = c;
-   }
-   else
-   {
-      if (w->main_client)
-      {
-	 c->prev = w->main_client;
-	 c->next = w->main_client->next;
-      } else {
-	 c->prev = w->head_client;
-	 c->next = w->head_client->next;
-       }	    
-      c->prev->next = c;
-      c->next->prev = c;
-   }
-
-#ifdef STANDALONE
-   c->backing      = None;
-#else
-   c->backing      = NULL;
-#endif
-
-   for (i=0; i<MSK_COUNT; i++)
-     c->backing_masks[i] = None;
-
-#if defined (USE_XFT)
-   c->xftdraw = NULL;
-#endif
-
-   c->flags     = 0;
-   c->icon      = None;
-   c->icon_mask = None;
-
-
-#ifndef REDUCE_BLOAT
-  c->icon_rgba_data = NULL;
-#endif
-
-  c->host_machine = NULL;
   if (XGetWMClientMachine(c->wm->dpy, c->window, &text_prop))
   {
     c->host_machine = strdup((char *) text_prop.value);
@@ -197,7 +192,8 @@ base_client_new(Wm *w, Window win)
     dbg("%s() got host machine for ewmh : %s\n", __func__, c->host_machine);
   }
   
-  c->pid = 0;
+  /* EWMH PID */
+
   if (XGetWindowProperty (w->dpy, win, 
 			  w->atoms[_NET_WM_PID],
 			  0, 2L,
@@ -214,9 +210,10 @@ base_client_new(Wm *w, Window win)
       dbg("%s() got ewmh pid : FAILED\n", __func__ );
     }
 
-  if (data) XFree(data);
-
   c->has_ping_protocol = False;
+  c->pings_pending     = -1;
+
+  if (data) XFree(data);
 
   /* We detect any errors here to check the window hasn't dissapeared half
    * way through. A bit hacky ...
@@ -224,8 +221,10 @@ base_client_new(Wm *w, Window win)
    misc_trap_xerrors();
 
    /* Check for 'special' extra button/ping protocols */
+
    if (XGetWMProtocols(c->wm->dpy, c->window, &protocols, &n)) 
      {
+       dbg("%s() checking wm protocols ( %i found )\n", __func__, n);
        for (i=0; i<n; i++)
 	 {
 	   if (protocols[i] == c->wm->atoms[_NET_WM_CONTEXT_HELP])
@@ -250,12 +249,31 @@ base_client_new(Wm *w, Window win)
 	       dbg("%s() has PING ewmh\n", __func__);
 	       c->has_ping_protocol = True;
 	     }
+#endif
+#ifdef USE_XSYNC
+	   else if (protocols[i] == c->wm->atoms[_NET_WM_SYNC_REQUEST])
+	     {
+	       c->has_ewmh_sync = True;
+	       dbg("%s() client has _NET_WM_SYNC_REQUEST\n", __func__);
+	     }
 #endif
 	 }
        XFree(protocols);
     }
 
-   c->pings_pending = -1;
+
+   /* We dont do workspaces! */
+
+   val[0] = 1;
+   XChangeProperty(w->dpy, c->window, w->atoms[_NET_WM_DESKTOP] ,
+		   XA_CARDINAL, 32, PropModeReplace,
+		   (unsigned char *)val, 1);
+   
+   ewmh_set_allowed_actions(w, c);
+
+   if (w->config->no_cursor && w->blank_curs)
+     XDefineCursor(w->dpy, c->window, w->blank_curs);
+
 
    client_set_state(c, WithdrawnState);
 
@@ -266,6 +284,8 @@ base_client_new(Wm *w, Window win)
        return NULL;
      }
 
+   list_add(&w->client_age_list, NULL, 0, (void*)c);
+
    return c;
 }
 
@@ -273,30 +293,46 @@ void
 base_client_process_name(Client *c)
 {
   Wm *w = c->wm;
-  XTextProperty text_prop;
-  Client *p; int i, max = 0; char *tmp_name = NULL;
 
-   dbg("%s() called, name is %s\n", __func__, c->name);
+  /* Crazily handle what to set the clients name too. 
+   * Note there is normally a check for utf8 name before this.
+   */
 
+  XTextProperty  text_prop;
+  Client        *p = NULL; 
+  int            i, max = 0; 
+  char          *tmp_name = NULL;
+
+  dbg("%s() called, name is %s\n", __func__, c->name);
+   
   if (c->name == NULL)
     {
       c->name_is_utf8 = False;
+      
       if (XGetWMName(w->dpy, c->window, &text_prop) != 0)
 	{
 	  dbg("%s() name is from XGetWMName\n", __func__ );
+
 	  c->name = strdup((char *) text_prop.value);
 	  XFree((char *) text_prop.value);
+
 	}
       else
 	{
 	  XFetchName(w->dpy, c->window, (char **)&c->name);
+
 	  if (c->name == NULL) 
 	    {
 	      XStoreName(w->dpy, c->window, "<unnamed>");
 	      XFetchName(w->dpy, c->window, (char **) &c->name);
+
 	      if (c->name == NULL) 
 		{
-		  /* something is seriously wrong if we get here */
+		  /* Something is seriously wrong if we get here 
+                   * Its likely the client window has dissapered. 
+		   */
+		  c->name = strdup("unknown"); /* something to free */
+		  
 		  dbg("%s() WARNING, name is still null after store/fetch\n",
 		      __func__ );
 		  return;
@@ -305,23 +341,24 @@ base_client_process_name(Client *c)
 	}
     }
   
-  /* If window name already exists, rename, adding <%i> to it */
-  if (w->head_client)
+  /* If window name already exists, rename, adding <%i> to it if app window */
+
+  if (!stack_empty(w) && c->type == MBCLIENT_TYPE_APP)
     {
-      START_CLIENT_LOOP(w,p);
-      
-      if (strncmp(p->name, c->name, strlen(c->name)) == 0
-	  && p != c)
+      stack_enumerate(w, p)
 	{
-	  if (strcmp(p->name, c->name) == 0)
+	  if (strncmp(p->name, c->name, strlen(c->name)) == 0
+	      && p != c)
 	    {
-	      if (!max) max = 1;
-	    } else {
-	      i = atoi(p->name+strlen(c->name)+2);
-	      if (i > max) max = i;
+	      if (strcmp(p->name, c->name) == 0)
+		{
+		  if (!max) max = 1;
+		} else {
+		  i = atoi(p->name+strlen(c->name)+2);
+		  if (i > max) max = i;
+		}
 	    }
 	}
-      END_CLIENT_LOOP(w,p);
       
       if (max)
 	{
@@ -366,7 +403,6 @@ base_client_reparent(Client *c)
    ;
 }
 
-
 /* redraw the clients frame */
 void
 base_client_redraw(Client *c, Bool use_cache)
@@ -374,7 +410,6 @@ base_client_redraw(Client *c, Bool use_cache)
    ;
 }
 
-
 /* button press on frame */
 void
 base_client_button_press(Client *c, XButtonEvent *e)
@@ -389,27 +424,12 @@ base_client_iconize(Client *c)
   c->hide(c);
 }
 
-
 /* move and resize the window */
 void
 base_client_move_resize(Client *c)
 {
   int i;
 
-#ifdef STANDALONE
-   if (c->backing != None)
-     {
-       XFreePixmap(c->wm->dpy, c->backing);
-       c->backing = None;
-     }
-#else
-   if (c->backing != NULL)
-     {
-       mb_drawable_unref(c->backing);
-       c->backing = NULL;
-     }
-#endif
-
    for (i=0; i<MSK_COUNT; i++)
      if (c->backing_masks[i] != None)
        {
@@ -428,169 +448,101 @@ base_client_get_coverage(Client *c, int *x, int *y, int *w, int *h)
   *x = c->x; *y = c->y;*w = c->width;*h = c->height;   
 }
 
-void
-base_client_hide_transients(Client *c)
-{
-   Client *t;
-   if (c == NULL) return;
-
-   for(t = c->prev; t != c; t = t->prev)
-     if (t->type == dialog && t->trans == c && t->mapped)
-       t->hide(t);
-
-   comp_engine_client_hide(c->wm, c);
-}
-
-
-void
+void 				/* TODO: Method not needed any more ? */
 base_client_hide(Client *c)
 {
-  base_client_hide_transients(c);
-
-  comp_engine_client_hide(c->wm, c);
+  ;
 }
 
 void
 base_client_show(Client *c)
 {
-
-   Client *t, *client_msg = NULL;
-
-   for(t = c->prev; t != c; t = t->prev)
-   {
-      switch (t->type)
-      {
-	 case dialog:    /* raise any transients */
-	    dbg("%s() Transient found, name: %s mapped: %i\n", __func__, t->name, t->mapped);
-	    if ((t->trans == c || t->trans == NULL) && t->mapped)
-	    {
-	       dbg("%s() raising transient %s\n", __func__, t->name);
-
-#ifdef USE_MSG_WIN
-	       /* Only raise 'global' msg wins above everything */
-	       if (t->flags & CLIENT_IS_MESSAGE_DIALOG
-		   && !(t->flags & CLIENT_IS_MESSAGE_DIALOG_HI)
-		   && !(t->flags & CLIENT_IS_MESSAGE_DIALOG_LO))
-		 {
-		   client_msg = t;
-		 }
-	       else
-#endif
-		 t->show(t);
-	    }
-	    /* raise any toolbar transients */
-	    else if (c->type != toolbar
-		     && t->trans != NULL
-		     && t->trans->type == toolbar
-		     && client_get_state(t->trans) == NormalState)
-	       t->show(t);
-	    break;
-      default:
-	    break;
-      }
-   }
-
-   comp_engine_client_show(c->wm, c);
-
-   /* Make sure message windows are _really_ on top */
-   if (client_msg) 
-     {
-       dbg("%s() really raising client message...", __func__);
-       client_msg->show(client_msg);
-       comp_engine_client_show(c->wm, client_msg);
-     }
-
-   ewmh_set_active(c->wm);
+  ;
 }
 
 void /* cb for this needed, or let wm handle it */
 base_client_destroy(Client *c)
 {
+  Wm *w = c->wm;
   int i = 0;
+  Client *p = NULL;
    /* Free its memory + remove from list */
 
-   Client *t = NULL, *prev_client = NULL;
    dbg("%s() called\n", __func__);
 
-   if (c == c->wm->prev_main_client)
-     c->wm->prev_main_client = NULL;
 
-   /* destroy any transients */
-   for(t = c->prev; t != c; t = prev_client)
+   /* Update focus list and anythink that is transient for this */
+   stack_enumerate(w, p)
      {
-       prev_client = t->prev;
-       if (t->type == dialog && t->trans == c)
-	 t->destroy(t);
+       if (p->next_focused_client == c)
+	 p->next_focused_client = c->next_focused_client;
+
+       /* TODO: It may be safer to destroy any transients automatically 
+        *       This is what we used to do. Its 'assumed' however the app
+        *       will do this its self. 
+        */
+
+       if (p->trans == c)
+#ifdef USE_ALT_INPUT_WIN
+	 {
+	   if (p->flags & (CLIENT_TB_ALT_TRANS_FOR_DIALOG|CLIENT_TB_ALT_TRANS_FOR_APP))
+	     p->hide(p);
+	   else
+	     p->trans = c->trans;
+	 }
+#else
+	 p->trans = c->trans;
+#endif
      }
 
 #ifdef USE_LIBSN
-   wm_sn_cycle_remove(c->wm, c->window);
+   wm_sn_cycle_remove(w, c->window);
 #endif       
 
    if (c->has_ping_protocol && c->pings_pending != -1) 
-     c->wm->n_active_ping_clients--;
+     w->n_active_ping_clients--;
 
-   /* remove from circular list */
-   if (c->prev != c)
-     {
-       if (c->wm->head_client == c) 
-	 c->wm->head_client = c->next;
-       if (c->prev != NULL) c->prev->next = c->next;
-       if (c->next != NULL) c->next->prev = c->prev;
-    } else {
-      c->wm->head_client = NULL;
-      if (c->type == mainwin) c->wm->main_client = NULL;
-    }
+   comp_engine_client_destroy(w, c);
+
+   list_remove(&w->client_age_list, (void*)c);
+
+   stack_remove(c);
+
+   /* Now free up various resources */
 
    if (c->type != MBCLIENT_TYPE_OVERRIDE)
      {
        client_buttons_delete_all(c);
        
-       ewmh_update(c->wm);
-   
-#if defined (USE_XFT)
-       if (c->xftdraw != NULL) XftDrawDestroy(c->xftdraw);
-#endif
+       ewmh_update(w);
 
        if (c->frame && c->frame != c->window) 
-	 XDestroyWindow(c->wm->dpy, c->frame);
-       if (c->title_frame != c->window 
-	   && c->title_frame != c->frame
-	   && c->title_frame != None) 
-	 XDestroyWindow(c->wm->dpy, c->title_frame);
-
-#ifdef STANDALONE
-       if (c->backing != None)
-	   XFreePixmap(c->wm->dpy, c->backing);
-#else
-       if (c->backing != NULL)
-	   mb_drawable_unref(c->backing);
-#endif
+	 {
+	   XDestroySubwindows(w->dpy, c->frame);
+	   XDestroyWindow(w->dpy, c->frame);
+	 }
 
        for (i=0; i<MSK_COUNT; i++)
 	 if (c->backing_masks[i] != None)
-	   XFreePixmap(c->wm->dpy, c->backing_masks[i]);
-       
-       if (c->icon != None && c->icon != c->wm->generic_icon)
-	 XFreePixmap(c->wm->dpy, c->icon);
-       if (c->icon_mask != None && c->icon_mask != c->wm->generic_icon_mask)
-	 XFreePixmap(c->wm->dpy, c->icon_mask);
-       
-#ifndef REDUCE_BLOAT
+	   XFreePixmap(w->dpy, c->backing_masks[i]);
+
+       /* No need to free up pixmap icon data client resource  */
+
        if (c->icon_rgba_data) XFree(c->icon_rgba_data);
-#endif
+
      }    
 
-    if (c->name) XFree(c->name);
-    if (c->startup_id) XFree(c->startup_id);
-    if (c->size) XFree(c->size);
+    if (c->name)         XFree(c->name);
+    if (c->startup_id)   XFree(c->startup_id);
+    if (c->size)         XFree(c->size);
     if (c->host_machine) free(c->host_machine);
 
-    comp_engine_client_hide(c->wm, c);
-    comp_engine_client_destroy(c->wm, c);
-
+    if (c == w->focused_client)
+      w->focused_client = NULL;
 
     free(c);
+
+
 }
 
 
