@@ -1,17 +1,22 @@
-/* matchbox - a lightweight window manager
-
-   Copyright 2002 Matthew Allum
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-*/
+/* 
+ *  Matchbox Window Manager - A lightweight window manager not for the
+ *                            desktop.
+ *
+ *  Authored By Matthew Allum <mallum@o-hand.com>
+ *
+ *  Copyright (c) 2002, 2004 OpenedHand Ltd - http://o-hand.com
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
 
 #include "ewmh.h"
 
@@ -92,9 +97,6 @@ ewmh_init(Wm *w)
     "_WIN_SUPPORTING_WM_CHECK",
 
     "_MB_WM_WINDOW_TYPE_MESSAGE",
-    "_MB_WM_WINDOW_TYPE_MESSAGE_TIMEOUT",
-    "_MB_WM_WINDOW_TYPE_MESSAGE_STATIC_0", /* Message wins which have the  */
-    "_MB_WM_WINDOW_TYPE_MESSAGE_STATIC_1", /* decor but not timing         */
 
     "_NET_WM_WINDOW_TYPE",
 
@@ -106,16 +108,20 @@ ewmh_init(Wm *w)
     "_NET_WM_WINDOW_TYPE_MENU",
     "CM_TRANSLUCENCY",
     "_MB_DOCK_TITLEBAR_SHOW_ON_DESKTOP",
-    "_MB_WIN_SUB_NAME",
     "_MB_GRAB_TRANSFER",
     "_NET_WM_WINDOW_TYPE_INPUT",
-    "INTEGER" 			/* XXX Needed ? */
-
+    "_NET_WM_STATE_ABOVE",
+    "WM_TRANSIENT_FOR",
+    "_NET_WM_SYNC_REQUEST_COUNTER",
+    "_NET_WM_SYNC_REQUEST"
   };
 
   XInternAtoms (w->dpy, atom_names, ATOM_COUNT,
                 False, w->atoms);
 
+#ifdef USE_XSYNC
+  ewmh_sync_init(w);
+#endif
 }
 
 void
@@ -145,68 +151,91 @@ ewmh_handle_root_message(Wm *w, XClientMessageEvent *e)
    dbg("%s() called\n", __func__);
 
    if (e->message_type == w->atoms[_NET_ACTIVE_WINDOW])
-   {
-      dbg("%s() got active window message for win %li", __func__, e->window);
-      if ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
-	{
-	  wm_activate_client(c);
-	}
-	
-      return 1;
-   } else if (e->message_type == w->atoms[_NET_CLOSE_WINDOW]) {
-	 if ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
-	    client_deliver_delete(c);
-	 return 1;
-   } else if (e->message_type == w->atoms[WM_PROTOCOLS]
-	      && e->data.l[0] == w->atoms[_NET_WM_PING]) {
-     if ((c = wm_find_client(w, e->data.l[1], WINDOW)) != NULL)
-       {
-	 dbg("%s() pong from %s\n", __func__, c->name);
+     {
+       dbg("%s() got active window message for win %li", __func__, e->window);
+       if ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
+	 wm_activate_client(c);
+       return 1;
+     } 
+   else if (e->message_type == w->atoms[_NET_CLOSE_WINDOW]) 
+     {
+       if ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
+	 client_deliver_delete(c);
+       return 1;
+     } 
+   else if (e->message_type == w->atoms[WM_PROTOCOLS]
+	    && e->data.l[0] == w->atoms[_NET_WM_PING]) 
+     {
+       if ((c = wm_find_client(w, e->data.l[1], WINDOW)) != NULL)
+	 {
+	   dbg("%s() pong from %s\n", __func__, c->name);
 
-	 /* We got a response to a ping. stop pinging it now
-            until close button is pressed again. 
-          */
-	 if (c->pings_pending > 0) 
-	   {
-	     c->pings_pending = -1;
-	     c->wm->n_active_ping_clients--;
-	   }
-       }
-   } else if (e->message_type == w->atoms[WINDOW_STATE]) {
-     if (e->data.l[1] == w->atoms[WINDOW_STATE_FULLSCREEN]
-	 && ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
-	 && c->type == mainwin)
-       {
-	 dbg("got EWMH fullscreen state change\n");
-	 switch (e->data.l[0])
-	   {
-	   case _NET_WM_STATE_REMOVE:
-	     if (c->flags & CLIENT_FULLSCREEN_FLAG)
+	   /* We got a response to a ping. stop pinging it now
+	      until close button is pressed again. 
+	   */
+	   if (c->pings_pending > 0) 
+	     {
+	       c->pings_pending = -1;
+	       w->n_active_ping_clients--;
+	     }
+	 }
+     } 
+   else if (e->message_type == w->atoms[WINDOW_STATE]) 
+     {
+       if (e->data.l[1] == w->atoms[WINDOW_STATE_FULLSCREEN]
+	   && ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
+	   && c->type == MBCLIENT_TYPE_APP)
+	 {
+	   dbg("got EWMH fullscreen state change\n");
+	   switch (e->data.l[0])
+	     {
+	     case _NET_WM_STATE_REMOVE:
+	       if (c->flags & CLIENT_FULLSCREEN_FLAG)
+		 main_client_toggle_fullscreen(c);
+	       break;
+	     case _NET_WM_STATE_ADD:
+	       if (!(c->flags & CLIENT_FULLSCREEN_FLAG))
+		 main_client_toggle_fullscreen(c);
+	       break;
+	     case _NET_WM_STATE_TOGGLE:
 	       main_client_toggle_fullscreen(c);
-	     break;
-	   case _NET_WM_STATE_ADD:
-	     if (!(c->flags & CLIENT_FULLSCREEN_FLAG))
-	       main_client_toggle_fullscreen(c);
-	     break;
-	   case _NET_WM_STATE_TOGGLE:
-	     main_client_toggle_fullscreen(c);
-	     break;
-	   }
-       }
-     return 1;
-
-   } else if (e->message_type == w->atoms[_NET_SHOW_DESKTOP]
-	      && wm_get_desktop(w) ) {
-     dbg("%s() got desktop message\n", __func__);
-     if (e->data.l[0] == 1)
-       { 			/* Show the desktop, if not shown */
-	 if (!(w->flags & DESKTOP_RAISED_FLAG))
-	   wm_toggle_desktop(w);
-       } else {                 /* Hide the desktop, if shown */
-	 if (w->flags & DESKTOP_RAISED_FLAG)
-	   wm_toggle_desktop(w);
-       }
-   } else { ; }
+	       break;
+	     }
+	 }
+       else if (e->data.l[1] == w->atoms[WINDOW_STATE_ABOVE]
+		&& ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
+		&& c->type == MBCLIENT_TYPE_DIALOG)
+	 {
+	   dbg("got EWMH above state change\n");
+	   switch (e->data.l[0])
+	     {
+	     case _NET_WM_STATE_REMOVE:
+	       c->flags &= ~CLIENT_HAS_ABOVE_STATE;
+	       break;
+	     case _NET_WM_STATE_ADD:
+	       c->flags |= CLIENT_HAS_ABOVE_STATE;
+	       break;
+	     case _NET_WM_STATE_TOGGLE:
+	       c->flags ^= CLIENT_HAS_ABOVE_STATE;
+	       break;
+	     }
+	   wm_activate_client(c);
+	 }
+       return 1;
+     } 
+   else if (e->message_type == w->atoms[_NET_SHOW_DESKTOP]
+	    && wm_get_desktop(w) ) 
+     {
+       dbg("%s() got desktop message\n", __func__);
+       if (e->data.l[0] == 1)
+	 { 			/* Show the desktop, if not shown */
+	   if (!(w->flags & DESKTOP_RAISED_FLAG))
+	     wm_toggle_desktop(w);
+	 } else {                 /* Hide the desktop, if shown */
+	   if (w->flags & DESKTOP_RAISED_FLAG)
+	     wm_toggle_desktop(w);
+	 }
+     }
    
    return 0;
 }
@@ -221,11 +250,13 @@ ewmh_update(Wm *w)
 void
 ewmh_update_lists(Wm *w)
 {
-   Client *c = NULL;
-   Window  *wins = NULL;
-   int num = 0, cnt = 0;
+   Client        *c = NULL;
+   Window        *wins = NULL;
+   int            cnt = 0;
    unsigned long  val[1];
 
+   dbg("%s(): called %i\n", __func__, n_stack_items(w)); 
+
 #ifdef USE_LIBSN
    unsigned char *bin_map_str = NULL; 
    int  bin_map_cnt = 0;
@@ -291,43 +322,50 @@ ewmh_update_lists(Wm *w)
 
 #endif
 
-   if (w->head_client)
-   {
-     dbg("%s(): updating ewmh list props\n", __func__ ) ;   
+  /* Root window client win lists */
 
-     START_CLIENT_LOOP(w,c);
-     {
-       num++;
-     }
-     END_CLIENT_LOOP(w,c);
+   if (!stack_empty(w))
+   {
+     MBList *item = NULL;
+
+     dbg("%s(): updating ewmh list props %i items\n", 
+	 __func__, n_stack_items(w) ) ;   
+
+     wins = malloc(sizeof(Window)*n_stack_items(w));
      
-     wins = malloc(sizeof(Window)*num);
-     
-     START_CLIENT_LOOP(w,c);
-     {
-       wins[cnt++] = c->window;
-     }
-     END_CLIENT_LOOP(w,c);
-  
-     XChangeProperty(w->dpy, w->root, w->atoms[_NET_CLIENT_LIST] ,
-		     XA_WINDOW, 32, PropModeReplace,
-		     (unsigned char *)wins, num);
+     stack_enumerate(w,c)
+       {
+	 wins[cnt++] = c->window;
+	 dbg("%s() adding %s\n", __func__, c->name);
+       }
   
      XChangeProperty(w->dpy, w->root, w->atoms[_NET_CLIENT_LIST_STACKING] ,
 		     XA_WINDOW, 32, PropModeReplace,
-		     (unsigned char *)wins, num);
+		     (unsigned char *)wins, n_stack_items(w));
+
+     /* Update _NET_CLIENT_LIST but with 'age' order rathe rthan stacking */
+
+     cnt = 0;
+
+     list_enumerate(w->client_age_list, item)
+       {
+	 c = (Client*)item->data;
+	 wins[cnt++] = c->window;
+	 dbg("%s() adding %s\n", __func__, c->name);
+       }
+
+     XChangeProperty(w->dpy, w->root, w->atoms[_NET_CLIENT_LIST] ,
+		     XA_WINDOW, 32, PropModeReplace,
+		     (unsigned char *)wins, n_stack_items(w));
    }
 
-   /* Desktop */
-   if (w->flags & DESKTOP_RAISED_FLAG)
-     val[0] = 1;
-   else
-     val[0] = 0;
+   /* Desktop showing hint */
+
+   val[0] = (w->flags & DESKTOP_RAISED_FLAG) ? 1 : 0;
 
    XChangeProperty(w->dpy, w->root, w->atoms[_NET_SHOW_DESKTOP],
 		   XA_CARDINAL, 32, PropModeReplace, 
 		   (unsigned char *)val, 1);
-
   if (wins)
     free(wins);
 }
@@ -349,6 +387,7 @@ ewmh_update_rects(Wm *w)
   if (w->flags & DESKTOP_DECOR_FLAG)
     {
       /* Desktop is decorated, needs to know frame border sizes  */
+
       val[0] += theme_frame_defined_width_get(w->mbtheme, 
 						   FRAME_MAIN_WEST );
       val[2] -= theme_frame_defined_width_get(w->mbtheme, 
@@ -365,36 +404,60 @@ ewmh_update_rects(Wm *w)
 		  XA_CARDINAL, 32, PropModeReplace, (unsigned char *)val, 4);
 }
 
+void 
+ewmh_state_set(Client *c)
+{
+  Wm   *w = c->wm;
+
+  Atom atom_states[2];
+  int  n_atom_states = 0;
+
+  /* We need to set ewmh state as some apps need to know when they 
+   * return from fullscreen
+   */
+
+  if (c->flags & CLIENT_FULLSCREEN_FLAG)
+    atom_states[n_atom_states++] = w->atoms[WINDOW_STATE_FULLSCREEN];
+
+  if  (c->flags & CLIENT_IS_MODAL_FLAG)
+    atom_states[n_atom_states++] = w->atoms[WINDOW_STATE_MODAL];
+
+  XChangeProperty(w->dpy, c->window, w->atoms[WINDOW_STATE],
+		  XA_ATOM, 32, PropModeReplace, 
+		  (unsigned char *)atom_states, n_atom_states);
+}
+
 Bool 
 ewmh_state_check(Client *c, Atom atom_state_wanted)
 {
-   unsigned long n;
-   unsigned long extra;
-   int format;
-   int status;
-   int i;
-   Atom realType;
-   Atom * value = NULL;
+  Wm   *w = c->wm;
 
-   status = XGetWindowProperty(c->wm->dpy, c->window,
-			       c->wm->atoms[WINDOW_STATE],
-			       0L, 1000000L,
-			       0, XA_ATOM, &realType, &format,
-			       &n, &extra, (unsigned char **) &value);
-   if (status == Success)
-     {
-       if (realType == XA_ATOM && format == 32 && n > 0)
-	 {
-	   for(i=0; i < n; i++)
-	     if (value[i] && value[i] == atom_state_wanted)
-	       {
-		 if (value) XFree(value);
-		 return True;
-	       }
-	 }
-     }
+  unsigned long n;
+  unsigned long extra;
+  int           format, status, i;
+  Atom          realType, *value = NULL;
+
+  status = XGetWindowProperty(w->dpy, c->window,
+			      w->atoms[WINDOW_STATE],
+			      0L, 1000000L,
+			      0, XA_ATOM, &realType, &format,
+			      &n, &extra, (unsigned char **) &value);
+  if (status == Success)
+    {
+      if (realType == XA_ATOM && format == 32 && n > 0)
+	{
+	  for(i=0; i < n; i++)
+	    if (value[i] && value[i] == atom_state_wanted)
+	      {
+		if (value) XFree(value);
+		return True;
+	      }
+	}
+    }
+  
+  if (value) 
+    XFree(value);
 
-   if (value) XFree(value);
    return False;
 }
 
@@ -408,10 +471,10 @@ ewmh_set_allowed_actions(Wm *w, Client *c)
     0, 0
   }; 
 
-  if (c->type == mainwin)
+  if (c->type == MBCLIENT_TYPE_APP)
     actions[num_actions++] = w->atoms[_NET_WM_ACTION_FULLSCREEN];
 
-  if (c->type == dock || c->type == dialog)
+  if (c->type == MBCLIENT_TYPE_PANEL || c->type == MBCLIENT_TYPE_DIALOG)
     actions[num_actions++] = w->atoms[_NET_WM_ACTION_MOVE];
 
   XChangeProperty(w->dpy, c->window, w->atoms[_NET_WM_ALLOWED_ACTIONS],
@@ -422,21 +485,13 @@ ewmh_set_allowed_actions(Wm *w, Client *c)
 void 
 ewmh_set_active(Wm *w)
 {
-  Client *c;
+  Client              *c;
   static unsigned long last_active; 
-  unsigned long  val[1];
+  unsigned long        val[1] = { 0 };
+
+  if ((c = wm_get_visible_main_client(w)) != NULL)
+    val[0] = c->window;
 
-  if ((w->flags & DESKTOP_RAISED_FLAG) && (c = wm_get_desktop(w)) != NULL)
-    { 
-      val[0] = c->window; 
-      dbg("%s() setting desktop ( %li ) as active\n", __func__, c->window );
-    }
-  else if (w->main_client)
-    { 
-      val[0] = w->main_client->window; 
-    }
-  else val[0] = 0;
-   
   if (last_active == val[0]) 	/* avoid the roundtrip if pos */
     return;
 
@@ -459,16 +514,16 @@ ewmh_hung_app_check(Wm *w)
 
   Client *c = NULL;
 
-  if (!w->head_client) return;
+  if (stack_empty(w)) return;
 
   dbg("%s() called\n", __func__ );
 
-  START_CLIENT_LOOP(w, c)
+  stack_enumerate(w, c)
     {
       if (c->has_ping_protocol && c->pings_pending != -1)
 	{
 	  XEvent e;
-
+	  
 	  c->pings_pending++;
 
 	  dbg("%s() pinging %s\n", __func__, c->name);
@@ -492,7 +547,6 @@ ewmh_hung_app_check(Wm *w)
 	    }
 	}
     }
-  END_CLIENT_LOOP(w, c);
 
 #endif
 }
@@ -562,12 +616,12 @@ static void set_supported(Wm *w) /*  */
 unsigned char *
 ewmh_get_utf8_prop(Wm *w, Window win, Atom req_atom)
 {
-  Atom type;
-  int format;
-  long bytes_after;
+  Atom           type;
+  int            format, result;
+  long           bytes_after, n_items;
   unsigned char *str = NULL;
-  long n_items;
-  int result;
+
+  misc_trap_xerrors();
 
   result =  XGetWindowProperty (w->dpy, win, req_atom,
 				0, 1024L,
@@ -575,7 +629,9 @@ ewmh_get_utf8_prop(Wm *w, Window win, Atom req_atom)
 				&type, &format, &n_items,
 				&bytes_after, (unsigned char **)&str);
 
-  if (result != Success || str == NULL)
+
+
+  if (misc_untrap_xerrors() || result != Success || str == NULL)
     {
       if (str) XFree (str);
       return NULL;
@@ -603,12 +659,12 @@ ewmh_get_utf8_prop(Wm *w, Window win, Atom req_atom)
 int*
 ewmh_get_icon_prop_data(Wm *w, Window win)
 {
-  Atom type;
-  int format;
-  long bytes_after;
+  Atom           type;
+  int            format, result;
+  long           bytes_after, n_items;
   unsigned char *data = NULL;
-  long n_items;
-  int result;
+
+  misc_trap_xerrors();
 
   result =  XGetWindowProperty (w->dpy, win, w->atoms[_NET_WM_ICON],
 				0, 100000L,
@@ -616,7 +672,7 @@ ewmh_get_icon_prop_data(Wm *w, Window win)
 				&type, &format, &n_items,
 				&bytes_after, (unsigned char **)&data);
 
-  if (result != Success || data == NULL)
+  if (misc_untrap_xerrors() || result != Success || data == NULL)
     {
       if (data) XFree (data);
       return NULL;
@@ -673,7 +729,205 @@ static void set_compliant(Wm *w) /* lets clients know were compliant (ish) */
    XStoreName(w->dpy, win, app_name);   
 }
 
-/* UTF 8 - borrowed from glib */
+
+/* 
+
+_NET_WM_SYNC_REQUEST
+
+This protocol uses the XSync extension the protocol specification and
+the library documentation<) to let client and window manager
+synchronize the repaint of the window manager frame and the client
+window. A client indicates that it is willing to participate in the
+protocol by listing _NET_WM_SYNC_REQUEST in the WM_PROTOCOLS property
+of the client window and storing the XID of an XSync counter in the
+property _NET_WM_SYNC_REQUEST_COUNTER. The initial value of this
+counter is not defined by this specification.
+
+A window manager uses this protocol by preceding a ConfigureNotify
+event sent to a client by a client message as follows:
+
+type = ClientMessage
+window = the respective client window
+message_type = WM_PROTOCOLS
+format = 32
+data.l[0] = _NET_WM_SYNC_REQUEST
+data.l[1] = timestamp
+data.l[2] = low 32 bits of the update request number
+data.l[3] = high 32 bits of the update request number
+other data.l[] elements = 0
+
+After receiving one or more such message/ConfigureNotify pairs, and
+having handled all repainting associated with the ConfigureNotify
+events, the client MUST set the _NET_WM_SYNC_REQUEST_COUNTER to the 64
+bit number indicated by the data.l[2] and data.l[3] fields of the last
+client message received.
+
+By using either the Alarm or the Await mechanisms of the XSync
+extension, the window manager can know when the client has finished
+handling the ConfigureNotify events. The window manager SHOULD not
+resize the window faster than the client can keep up.
+
+The update request number in the client message is determined by the
+window manager subject to the restriction that it MUST NOT be 0. The
+number is generally intended to be incremented by one for each message
+sent. Since the initial value of the XSync counter is not defined by
+this specification, the window manager MAY set the value of the XSync
+counter at any time, and MUST do so when it first manages a new
+window.
+
+*/
+
+#ifdef USE_XSYNC
+
+static void
+sync_value_increment (XSyncValue *value)
+{
+  XSyncValue one;
+  int overflow;
+  
+  XSyncIntToValue (&one, 1);
+  XSyncValueAdd (value, *value, one, &overflow);
+}
+
+void
+ewmh_sync_init(Wm *w)
+{
+  if (!XSyncQueryExtension (w->dpy,
+                            &w->sync_event_base,
+                            &w->sync_error_base))
+    {
+      dbg("%s() XSyncQueryExtension FAILED.\n", __func__);
+      w->have_xsync = False;
+      return;
+    }
+
+  w->have_xsync = True;
+}
+
+void
+ewmh_sync_handle_event(Wm *w, XSyncAlarmNotifyEvent *ev)
+{
+  Client *client = NULL;
+
+  stack_enumerate(w, client)
+    {
+      if (client->ewmh_sync_alarm == ev->alarm)
+	break;
+    }
+
+  if (client) 			/* XXX should check alarm matches */
+    {
+      dbg("%s() found client %s\n", __func__, client->name);
+      client->move_resize(client);
+    }
+}
+
+Bool
+ewmh_sync_client_move_resize(Client *client)
+{
+  Wm *w = client->wm;
+  unsigned long highval, lowval;
+
+  if (!w->have_xsync) 
+    return False;
+  
+  if (!client->has_ewmh_sync)
+    return False;
+
+  if (client->ewmh_sync_is_waiting)
+    return False; 		/* XXX WRONG XXX */
+
+  ewmh_sync_client_init_counter(client);
+
+  lowval  = XSyncValueLow32 (client->ewmh_sync_value);
+  highval = XSyncValueHigh32 (client->ewmh_sync_value);
+
+  sync_value_increment (&client->ewmh_sync_value);
+
+  dbg("%s() delivering _NET_WM_SYNC_REQUEST\n", __func__);
+
+  client_deliver_message(client, 
+			 w->atoms[WM_PROTOCOLS], 
+			 w->atoms[_NET_WM_SYNC_REQUEST], 
+			 CurrentTime, lowval, highval,
+			 0);
+  
+  client->ewmh_sync_is_waiting = True;
+  
+  return True;
+}
+
+Bool
+ewmh_sync_client_init_counter(Client *client)
+{
+  Wm *w = client->wm;
+
+  XSyncAlarmAttributes values;
+  Atom                 type;
+  int                  format, result;
+  long                 bytes_after, n_items;
+  XID                 *value;
+
+  if (!w->have_xsync) 
+    return False;
+  
+  if (!client->has_ewmh_sync)
+    return False;
+
+  result =  XGetWindowProperty (w->dpy, client->window, 
+				w->atoms[_NET_WM_SYNC_REQUEST_COUNTER],
+				0, 1024L,
+				False, XA_CARDINAL,
+				&type, &format, &n_items,
+				&bytes_after, (unsigned char **)&value);
+
+  if (result != Success || value == NULL || format != 32)
+    {
+      dbg("%s() _NET_WM_SYNC_REQUEST_COUNTER failed\n", __func__);
+      if (value) XFree (value);
+      return False;
+    }
+
+  dbg("%s() creating alarm\n", __func__);
+
+  client->ewmh_sync_counter = *value;
+
+  XSyncIntsToValue (&client->ewmh_sync_value, random(), 0);
+  XSyncSetCounter (w->dpy, client->ewmh_sync_counter, client->ewmh_sync_value);
+
+  sync_value_increment (&client->ewmh_sync_value);
+
+  values.events = True;
+  values.trigger.counter    = client->ewmh_sync_counter;
+  values.trigger.wait_value = client->ewmh_sync_value;
+  values.trigger.value_type = XSyncAbsolute;
+  values.trigger.test_type  = XSyncPositiveComparison;
+  XSyncIntToValue (&values.delta, 1);
+  values.events = True;
+
+  /* Note that by default, the alarm increments the trigger value
+   * when it fires until the condition (counter.value < trigger.value)
+   * is FALSE again.
+   */
+  client->ewmh_sync_alarm = XSyncCreateAlarm (w->dpy,
+					      XSyncCACounter 
+					      | XSyncCAValue 
+					      | XSyncCAValueType 
+					      | XSyncCATestType 
+					      | XSyncCADelta 
+					      | XSyncCAEvents,
+					      &values);
+  XSync (w->dpy, False);
+
+  /* XXX untrap error here */
+
+  return True;
+}
+
+#endif
+
+
+/* UTF8 - borrowed from glib. XXX fontconfig may actually provide these calls*/
 
 #define UTF8_COMPUTE(Char, Mask, Len)                                         \
   if (Char < 128)                                                             \
