@@ -1,17 +1,22 @@
-/* matchbox - a lightweight window manager
-
-   Copyright 2002 Matthew Allum
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-*/
+/* 
+ *  Matchbox Window Manager - A lightweight window manager not for the
+ *                            desktop.
+ *
+ *  Authored By Matthew Allum <mallum@o-hand.com>
+ *
+ *  Copyright (c) 2002, 2004 OpenedHand Ltd - http://o-hand.com
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
 
 #ifndef _STRUCTS_H_
 #define _STRUCTS_H_
@@ -47,15 +52,6 @@
 #include <langinfo.h>
 #endif
 
-#ifdef USE_PANGO
-#include <pango/pango.h>
-#include <pango/pangoxft.h>
-#endif
-
-#ifdef USE_XSETTINGS
-#include <xsettings-client.h>
-#endif 
-
 #ifdef USE_LIBSN
 #define SN_API_NOT_YET_FROZEN 1
 #define MB_SN_APP_TIMEOUT 30 	/* 30 second timeout for app startup */
@@ -73,34 +69,33 @@
 #include <X11/extensions/Xrender.h>
 #endif
 
+#ifdef USE_XSYNC
+#include <X11/extensions/sync.h>
+#endif
+
 #define GENERIC_ICON        PKGDATADIR "/mbnoapp.xpm"
 
 #ifdef MB_HAVE_PNG
 
-#define DEFAULT_THEME       DATADIR "/themes/blondie/matchbox/theme.xml"
+#define DEFAULT_THEME       DATADIR "/themes/Default/matchbox/theme.xml"
 
-#define DEFAULTTHEME       DATADIR "/themes/blondie/matchbox/theme.xml"
+#define DEFAULTTHEME       DATADIR "/themes/Default/matchbox/theme.xml"
 
-#define DEFAULT_THEMENAME  "blondie"
+#define DEFAULT_THEMENAME  "Default"
 
-#define DEFAULTTHEMENAME  "blondie"
+#define DEFAULTTHEMENAME  "Default"
 
 #else
 
-#define DEFAULT_THEME       DATADIR "/themes/bluebox/matchbox/theme.xml"
-#define DEFAULT_THEME_NAME  "bluebox"
+#define DEFAULT_THEME       DATADIR "/themes/Default/matchbox/theme.xml"
+#define DEFAULT_THEME_NAME  "Default"
 
 #endif
 
 #define CONFDEFAULTS       PKGDATADIR "/defaults"
 
-#define USE_EXTRAS 1
-
 /* Simple Macros  */
 
-#define START_CLIENT_LOOP(w,c) (c) = (w)->head_client; do {
-#define END_CLIENT_LOOP(w,c)   } while (((c) = (c)->next) \
-				        && ((c) != (w)->head_client) );
 #define MBMAX(x,y) ((x>y)?(x):(y))
 
 #ifdef DEBUG
@@ -111,10 +106,10 @@
 
 /* General Defines */
 
-#define NORTH  1
-#define EAST   2
-#define SOUTH  3
-#define WEST   4
+#define EAST   0
+#define SOUTH  1
+#define WEST   2
+#define NORTH  3     /* Note, North must be last or pixmap caching will fail*/
 
 #define ACTIVE   1
 #define INACTIVE 2
@@ -125,6 +120,8 @@
 #define MENU_ENTRY_PADDING 6
 #define MENU_ICON_PADDING 4
 
+#define N_DECOR_FRAMES 4
+
 /* Shadow defaults, only used with composite */
 
 #define SHADOW_RADIUS 6
@@ -194,9 +191,6 @@ enum {
   _NET_STARTUP_ID,
   _WIN_SUPPORTING_WM_CHECK,
   WINDOW_TYPE_MESSAGE,
-  WINDOW_TYPE_MESSAGE_TIMEOUT,
-  WINDOW_TYPE_MESSAGE_STATIC_0,
-  WINDOW_TYPE_MESSAGE_STATIC_1,
   WINDOW_TYPE,
   _NET_WM_PING,
   _NET_WM_PID,
@@ -206,10 +200,12 @@ enum {
   WINDOW_TYPE_MENU,
   CM_TRANSLUCENCY,
   MB_DOCK_TITLEBAR_SHOW_ON_DESKTOP,
-  MB_WIN_SUB_NAME,
   MB_GRAB_TRANSFER,
   WINDOW_TYPE_INPUT,
-  INTEGER,
+  WINDOW_STATE_ABOVE,
+  WM_TRANSIENT_FOR,
+  _NET_WM_SYNC_REQUEST_COUNTER,
+  _NET_WM_SYNC_REQUEST,
   ATOM_COUNT
 
 } MBAtomEnum;
@@ -225,13 +221,14 @@ enum {
 
 typedef enum 
 { 
-  dialog,
-  toolbar,
-  dock,
-  menu,
-  mainwin,
-  desktop,
-  MBCLIENT_TYPE_OVERRIDE
+  MBCLIENT_TYPE_DIALOG    = (1<<1),
+  MBCLIENT_TYPE_TOOLBAR   = (1<<2),
+  MBCLIENT_TYPE_PANEL     = (1<<3),
+  MBCLIENT_TYPE_TASK_MENU = (1<<4),
+  MBCLIENT_TYPE_APP       = (1<<5),
+  MBCLIENT_TYPE_DESKTOP   = (1<<6),
+  MBCLIENT_TYPE_OVERRIDE  = (1<<7),
+  MBCLIENT_TYPE_ANY       = (1<<8)
 
 } MBClientTypeEnum;
 
@@ -283,6 +280,14 @@ typedef struct _mb_client_button
 #define CLIENT_DOCK_TITLEBAR_SHOW_ON_DESKTOP (1<<20)
 #define CLIENT_IS_MINIMIZED    (1<<23) /* used by toolbars */
 #define CLIENT_TOOLBARS_MOVED_FOR_FULLSCREEN (1<<24)
+#define CLIENT_IS_TRANSIENT_FOR_ROOT (1<<25)
+#define CLIENT_HAS_URGENCY_FLAG (1<<26)
+#define CLIENT_HAS_ABOVE_STATE (1<<27)
+
+#ifdef USE_ALT_INPUT_WIN
+#define CLIENT_TB_ALT_TRANS_FOR_DIALOG (1<<28)
+#define CLIENT_TB_ALT_TRANS_FOR_APP    (1<<29)
+#endif
 
 /* Main Client structure */
 
@@ -293,19 +298,12 @@ typedef struct _client
 
   /* Window identification / title stuff */
 
-#ifdef USE_PANGO
+#if USE_XFT
   unsigned char    *name;
-  XftDraw          *xftdraw;
-  PangoLayout      *pgo_layout;
-#elif USE_XFT
-  unsigned char    *name;
-  XftDraw          *xftdraw;
 #else
   char	           *name;
 #endif
 
-  unsigned char    *subname ;             /* For optional subname title prop */
-
   int               name_rendered_end_pos;  /* used by theme engine */
   int               name_total_width;
 
@@ -351,19 +349,16 @@ typedef struct _client
 
   /* Decoration etc */
 
-  Window	    frame, title_frame;
-
-#ifdef STANDALONE
-  Pixmap            backing;
-#else
-  MBDrawable       *backing;
-#endif
-
+  Window	    frame, frames_decor[N_DECOR_FRAMES];
   Pixmap            backing_masks[MSK_COUNT];
 
   Bool              have_cache, have_set_bg;
   struct list_item *buttons; 
 
+  /* InputOnly modal 'blocker' win */
+
+  Window            win_modal_blocker;
+
   /* State stuff */
 
   int		    ignore_unmap;
@@ -376,10 +371,22 @@ typedef struct _client
   char             *host_machine;
   pid_t             pid;
 
+#ifdef USE_XSYNC
+  Bool              has_ewmh_sync;
+  XSyncCounter      ewmh_sync_counter;
+  XSyncValue        ewmh_sync_value;
+  XSyncAlarm        ewmh_sync_alarm;
+  Bool              ewmh_sync_is_waiting;
+#endif
+
   /* References */
    
   struct _wm       *wm;
-  struct _client   *prev, *next;
+
+  /* New */
+
+  struct _client   *above, *below;
+  struct _client   *next_focused_client;
 
   /* Client methods */
   
@@ -393,6 +400,10 @@ typedef struct _client
   void (* show)( struct _client* c );
   void (* iconize)( struct _client* c );
   void (* destroy)( struct _client* c );
+
+  /* new */
+  void (* raise)( struct _client* c );
+  void (* unmap)( struct _client* c );
    
 } Client;
 
@@ -442,6 +453,7 @@ typedef struct _kbdconfig
 
 enum {
   WM_DIALOGS_STRATERGY_CONSTRAINED,
+  WM_DIALOGS_STRATERGY_STATIC,
   WM_DIALOGS_STRATERGY_CONSTRAINED_HORIZ,
   WM_DIALOGS_STRATERGY_FREE,
 };
@@ -466,8 +478,6 @@ typedef struct _wm_config
   int           shadow_style;
   unsigned char shadow_color[4];
 
-
-
 #endif
    
   Time         dbl_click_time;
@@ -475,7 +485,7 @@ typedef struct _wm_config
   char        *ping_handler;
   MBConfigKbd *kb;
 
-  Bool         awt_workaround;
+  char        *force_dialogs;
 } Wm_config;
 
 
@@ -496,17 +506,6 @@ typedef struct _sn_execmapping_item
 } SnExecMappingItem;
 #endif
 
-#ifdef USE_MSG_WIN
-typedef struct MsgWinQueue
-{
-  Window              win;
-  int                 timeout;
-  struct MsgWinQueue *next;
-
-} MsgWinQueue;
-#endif
-
-
 /* WM Struct state flags XXX enum ? */
 
 #define MENU_FLAG           (1<<1)
@@ -522,6 +521,8 @@ typedef struct MsgWinQueue
 #define DEBUG_COMPOSITE_VISIBLE_FLAG (1<<9)
 #endif
 
+typedef struct list_item MBList; 
+
 /* Main WM struct  */
 
 typedef struct _wm
@@ -538,10 +539,18 @@ typedef struct _wm
   /* State stuff */
 
   int               flags;
-  Client*           head_client;    /* For ptr to beginning of list   */
   Client*           focused_client; /* currently focused client       */
-  Client*           main_client;    /* currently viewable main client */
-  Client*           prev_main_client; /* used for decorated desktop */
+
+  /* Stack stuff */
+
+  Client           *stack_top, *stack_bottom;
+  int               stack_n_items;     
+  Client           *stack_top_app; 
+  Client           *client_desktop;
+
+  MBList           *client_age_list; /* List of clients ordered by age */
+
+  /*******************/
 
   Wm_config        *config;  
 
@@ -551,12 +560,13 @@ typedef struct _wm
 
   /* Theme / visual look related date */
 
-  Cursor            curs, curs_busy, curs_drag;
+  Cursor            curs, curs_busy, curs_drag, blank_curs;
 
   struct _mbtheme  *mbtheme;
   Pixmap            generic_icon, generic_icon_mask; 
 
   Client           *have_titlebar_panel;
+  XColor            grey_col; 	/* Used for window backgrounds.*/
 
   /* Extra data need for various compile time opts */
 
@@ -578,15 +588,6 @@ typedef struct _wm
   XSettingsClient  *xsettings_client;
 #endif 
 
-#ifdef USE_MSG_WIN
-  MsgWinQueue      *msg_win_queue_head;
-#endif
-
-#ifdef USE_PANGO
-  PangoContext     *pgo;
-  PangoFontMap     *pgo_fontmap;
-#endif
-
 #ifdef USE_GCONF
   GConfClient      *gconf_client;
   GMainContext     *gconf_context;
@@ -629,12 +630,18 @@ typedef struct _wm
 
 #endif
 
+#ifdef USE_XSYNC
+  Bool              have_xsync;
+  int               sync_event_base;
+  int               sync_error_base;
+#endif
+
 #ifdef STANDALONE
-  Bool          have_toolbar_panel;
-  int           toolbar_panel_x;
-  int           toolbar_panel_y;
-  int           toolbar_panel_w;
-  int           toolbar_panel_h;
+  Bool             have_toolbar_panel;
+  int              toolbar_panel_x;
+  int              toolbar_panel_y;
+  int              toolbar_panel_w;
+  int              toolbar_panel_h;
 #endif
 
   int n_active_ping_clients; 	/* Number of apps we are pinging */
